/*******************************************************************************
 *  Imixs Workflow 
 *  Copyright (C) 2001, 2011 Imixs Software Solutions GmbH,  
 *  http://www.imixs.com
 *  
 *  This program is free software; you can redistribute it and/or 
 *  modify it under the terms of the GNU General Public License 
 *  as published by the Free Software Foundation; either version 2 
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful, 
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 *  General Public License for more details.
 *  
 *  You can receive a copy of the GNU General Public
 *  License at http://www.gnu.org/licenses/gpl.html
 *  
 *  Project: 
 *  	http://www.imixs.org
 *  	http://java.net/projects/imixs-workflow
 *  
 *  Contributors:  
 *  	Imixs Software Solutions GmbH - initial API and implementation
 *  	Ralph Soika - Software Developer
 *******************************************************************************/

package org.imixs.workflow.jaxrs;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.ejb.EJB;
import javax.ejb.Stateless;
import javax.ws.rs.Consumes;
import javax.ws.rs.DefaultValue;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.StreamingOutput;
import javax.ws.rs.core.UriInfo;

import org.imixs.workflow.ItemCollection;
import org.imixs.workflow.xml.EntityCollection;
import org.imixs.workflow.xml.XMLItemCollection;
import org.imixs.workflow.xml.XMLItemCollectionAdapter;

/**
 * The WorkflowService Handler supports methods to process different kind of
 * request URIs
 * 
 * @author rsoika
 * 
 */
@Path("/workflow")
@Produces({ "text/html", "application/xml", "application/json" })
@Stateless
public class WorkflowRestService {

	@EJB
	org.imixs.workflow.jee.ejb.WorkflowService workflowService;

	@GET
	@Produces("text/html")
	@Path("/")
	public StreamingOutput getHelpHTML() {

		return new StreamingOutput() {
			public void write(OutputStream out) throws IOException,
					WebApplicationException {

				out.write("<html><head>".getBytes());
				out.write("<style>".getBytes());
				out.write("table {padding:0px;width: 100%;margin-left: -2px;margin-right: -2px;}"
						.getBytes());
				out.write("body,td,select,input,li {font-family: Verdana, Helvetica, Arial, sans-serif;font-size: 13px;}"
						.getBytes());
				out.write("table th {color: white;background-color: #bbb;text-align: left;font-weight: bold;}"
						.getBytes());

				out.write("table th,table td {font-size: 12px;}".getBytes());

				out.write("table tr.a {background-color: #ddd;}".getBytes());

				out.write("table tr.b {background-color: #eee;}".getBytes());

				out.write("</style>".getBytes());
				out.write("</head><body>".getBytes());

				// body
				out.write("<h1>Imixs Workflow REST Service</h1>".getBytes());
				out.write("<p>Read the Imixs REST Service <a href=\"http://doc.imixs.org/xml/restservice.html\">Online Help</a> for a detailed description of this Service.</p>"
						.getBytes());
				out.write("<p>See the <a href=\"http://www.imixs.org\">Imixs Workflow Project Site</a> for general informations.</p>"
						.getBytes());

				// end
				out.write("</body></html>".getBytes());
			}
		};

	}

	/**
	 * Returns a collection of workitems representing the worklist by the
	 * current user
	 * 
	 * @param start
	 * @param count
	 * @param type
	 * @param sortorder
	 */
	@GET
	@Path("/worklist")
	public EntityCollection getWorkList(
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count,
			@QueryParam("type") String type,
			@DefaultValue("0") @QueryParam("sortorder") int sortorder,
			@QueryParam("items") String items) {
		Collection<ItemCollection> col = null;
		try {

			col = workflowService.getWorkList(null, start, count, type,
					sortorder);
			return XMLItemCollectionAdapter.putCollection(col,getItemList(items));

		} catch (Exception e) {
			e.printStackTrace();
		}
		return new EntityCollection();
	}

	/**
	 * 
	 * @param start
	 * @param count
	 * @param type
	 * @param sortorder
	 * @return
	 */
	@GET
	@Path("/worklist.xml")
	@Produces("application/xml")
	public EntityCollection getWorkListXML(
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count,
			@QueryParam("type") String type,
			@DefaultValue("0") @QueryParam("sortorder") int sortorder,
			@QueryParam("items") String items) {
		return getWorkList(start, count, type, sortorder,items);
	}
	
	@GET
	@Path("/worklist.json")
	@Produces("application/json")
	public EntityCollection getWorkListJSON(
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count,
			@QueryParam("type") String type,
			@DefaultValue("0") @QueryParam("sortorder") int sortorder,
			@QueryParam("items") String items) {
		return getWorkList(start, count, type, sortorder,items);
	}

	@GET
	@Path("/worklistbycreator")
	public EntityCollection getWorkListByCreator(
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count,
			@QueryParam("type") String type,
			@DefaultValue("0") @QueryParam("sortorder") int sortorder,
			@QueryParam("items") String items) {
		Collection<ItemCollection> col = null;
		try {

			col = workflowService.getWorkListByCreator(null, start, count,
					type, sortorder);
			return XMLItemCollectionAdapter.putCollection(col,getItemList(items));

		} catch (Exception e) {
			e.printStackTrace();
		}
		return new EntityCollection();
	}

	@GET
	@Path("/worklistbycreator.xml")
	@Produces("application/xml")
	public EntityCollection getWorkListByCreatorXML(
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count,
			@QueryParam("type") String type,
			@DefaultValue("0") @QueryParam("sortorder") int sortorder,
			@QueryParam("items") String items) {
		return getWorkListByCreator(start, count, type, sortorder,items);
	}
	
	@GET
	@Path("/worklistbycreator.json")
	@Produces("application/json")
	public EntityCollection getWorkListByCreatorJSON(
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count,
			@QueryParam("type") String type,
			@DefaultValue("0") @QueryParam("sortorder") int sortorder,
			@QueryParam("items") String items) {
		return getWorkListByCreator(start, count, type, sortorder,items);
	}
	

	@GET
	@Path("/worklistbyprocessid/{processid}")
	public EntityCollection getWorkListByProcessID(
			@PathParam("processid") int processid,
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count,
			@QueryParam("type") String type,
			@DefaultValue("0") @QueryParam("sortorder") int sortorder,
			@QueryParam("items") String items) {
		Collection<ItemCollection> col = null;
		try {
			col = workflowService.getWorkListByProcessID(processid, start,
					count, type, sortorder);
			return XMLItemCollectionAdapter.putCollection(col,getItemList(items));
		} catch (Exception e) {
			e.printStackTrace();
		}
		return new EntityCollection();
	}

	@GET
	@Path("/worklistbyprocessid/{processid}.xml")
	@Produces("application/xml")
	public EntityCollection getWorkListByProcessIDXML(
			@PathParam("processid") int processid,
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count,
			@QueryParam("type") String type,
			@DefaultValue("0") @QueryParam("sortorder") int sortorder,
			@QueryParam("items") String items) {
		return getWorkListByProcessID(processid, start, count, type, sortorder,items);
	}
	
	@GET
	@Path("/worklistbyprocessid/{processid}.json")
	@Produces("application/json")
	public EntityCollection getWorkListByProcessIDJSON(
			@PathParam("processid") int processid,
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count,
			@QueryParam("type") String type,
			@DefaultValue("0") @QueryParam("sortorder") int sortorder,
			@QueryParam("items") String items) {
		return getWorkListByProcessID(processid, start, count, type, sortorder,items);
	}
	

	@GET
	@Path("/worklistbygroup/{processgroup}")
	public EntityCollection getWorkListByGroup(
			@PathParam("processgroup") String processgroup,
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count,
			@QueryParam("type") String type,
			@DefaultValue("0") @QueryParam("sortorder") int sortorder,
			@QueryParam("items") String items) {
		Collection<ItemCollection> col = null;
		try {
			col = workflowService.getWorkListByGroup(processgroup, start,
					count, type, sortorder);
			return XMLItemCollectionAdapter.putCollection(col,getItemList(items));
		} catch (Exception e) {
			e.printStackTrace();
		}
		return new EntityCollection();
	}

	@GET
	@Path("/worklistbygroup/{processgroup}.xml")
	@Produces("application/xml")
	public EntityCollection getWorkListByGroupXML(
			@PathParam("processgroup") String processgroup,
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count,
			@QueryParam("type") String type,
			@DefaultValue("0") @QueryParam("sortorder") int sortorder,
			@QueryParam("items") String items) {
		return getWorkListByGroup(processgroup, start, count, type, sortorder,items);
	}
	
	@GET
	@Path("/worklistbygroup/{processgroup}.json")
	@Produces("application/json")
	public EntityCollection getWorkListByGroupJSON(
			@PathParam("processgroup") String processgroup,
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count,
			@QueryParam("type") String type,
			@DefaultValue("0") @QueryParam("sortorder") int sortorder,
			@QueryParam("items") String items) {
		return getWorkListByGroup(processgroup, start, count, type, sortorder,items);
	}
	

	@GET
	@Path("/worklistbyowner/{owner}")
	public EntityCollection getWorkListByOwner(
			@PathParam("owner") String owner,
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count,
			@QueryParam("type") String type,
			@DefaultValue("0") @QueryParam("sortorder") int sortorder,
			@QueryParam("items") String items) {
		Collection<ItemCollection> col = null;
		try {
			if ("null".equalsIgnoreCase(owner))
				owner=null;
			
			col = workflowService.getWorkListByOwner(owner, start, count, type,
					sortorder);
			return XMLItemCollectionAdapter.putCollection(col,getItemList(items));
		} catch (Exception e) {
			e.printStackTrace();
		}
		return new EntityCollection();
	}

	@GET
	@Path("/worklistbyowner/{owner}.xml")
	@Produces("application/xml")
	public EntityCollection getWorkListByOwnerXML(
			@PathParam("owner") String owner,
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count,
			@QueryParam("type") String type,
			@DefaultValue("0") @QueryParam("sortorder") int sortorder,
			@QueryParam("items") String items) {
		return getWorkListByOwner(owner, start, count, type, sortorder,items);
	}
	

	@GET
	@Path("/worklistbyowner/{owner}.json")
	@Produces("application/json")
	public EntityCollection getWorkListByOwnerJSON(
			@PathParam("owner") String owner,
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count,
			@QueryParam("type") String type,
			@DefaultValue("0") @QueryParam("sortorder") int sortorder,
			@QueryParam("items") String items) {
		return getWorkListByOwner(owner, start, count, type, sortorder,items);
	}
	

	@GET
	@Path("/worklistbywriteaccess")
	public EntityCollection getWorkListByWriteAccess(
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count,
			@QueryParam("type") String type,
			@DefaultValue("0") @QueryParam("sortorder") int sortorder,
			@QueryParam("items") String items) {
		Collection<ItemCollection> col = null;
		try {
			col = workflowService.getWorkListByWriteAccess(start, count, type,
					sortorder);
			return XMLItemCollectionAdapter.putCollection(col,getItemList(items));
		} catch (Exception e) {
			e.printStackTrace();
		}
		return new EntityCollection();
	}

	@GET
	@Path("/worklistbywriteaccess.xml")
	@Produces("application/xml")
	public EntityCollection getWorkListByWriteAccessXML(
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count,
			@QueryParam("type") String type,
			@DefaultValue("0") @QueryParam("sortorder") int sortorder,
			@QueryParam("items") String items) {

		return getWorkListByWriteAccess(start, count, type, sortorder,items);
	}
	
	@GET
	@Path("/worklistbywriteaccess.json")
	@Produces("application/json")
	public EntityCollection getWorkListByWriteAccessJSON(
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count,
			@QueryParam("type") String type,
			@DefaultValue("0") @QueryParam("sortorder") int sortorder,
			@QueryParam("items") String items) {

		return getWorkListByWriteAccess(start, count, type, sortorder,items);
	}
	

	@GET
	@Path("/worklistbyref/{uniqueid}")
	public EntityCollection getWorkListByRef(
			@PathParam("uniqueid") String uniqueid,
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count,
			@QueryParam("type") String type,
			@DefaultValue("0") @QueryParam("sortorder") int sortorder,
			@QueryParam("items") String items) {
		Collection<ItemCollection> col = null;
		try {
			col = workflowService.getWorkListByRef(uniqueid, start, count,
					type, sortorder);
			return XMLItemCollectionAdapter.putCollection(col,getItemList(items));
		} catch (Exception e) {
			e.printStackTrace();
		}
		return new EntityCollection();
	}

	@GET
	@Path("/worklistbyref/{uniqueid}.xml")
	@Produces("application/xml")
	public EntityCollection getWorkListByRefXML(
			@PathParam("uniqueid") String uniqueid,
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count,
			@QueryParam("type") String type,
			@DefaultValue("0") @QueryParam("sortorder") int sortorder,
			@QueryParam("items") String items) {
		return getWorkListByRef(uniqueid, start, count, type, sortorder,items);
	}
	
	@GET
	@Path("/worklistbyref/{uniqueid}.json")
	@Produces("application/json")
	public EntityCollection getWorkListByRefJSON(
			@PathParam("uniqueid") String uniqueid,
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count,
			@QueryParam("type") String type,
			@DefaultValue("0") @QueryParam("sortorder") int sortorder,
			@QueryParam("items") String items) {
		return getWorkListByRef(uniqueid, start, count, type, sortorder,items);
	}

	/**
	 * returns a singel workitem defined by $uniqueid
	 * 
	 * @param uniqueid
	 * @return
	 */
	@GET
	@Path("/workitem/{uniqueid}")
	public XMLItemCollection getWorkItem(
			@PathParam("uniqueid") String uniqueid,
			@QueryParam("items") String items) {
		ItemCollection workitem;
		try {
			workitem = workflowService.getWorkItem(uniqueid);
			return XMLItemCollectionAdapter.putItemCollection(workitem,getItemList(items));
		} catch (Exception e) {
			e.printStackTrace(); 
			return null;
		}
	}

	@GET
	@Path("/workitem/{uniqueid}.xml")
	@Produces("application/xml")
	public XMLItemCollection getWorkItemXML(
			@PathParam("uniqueid") String uniqueid,
			@QueryParam("items") String items) {
		return getWorkItem(uniqueid,items);
	}
	
	@GET
	@Path("/workitem/{uniqueid}.json")
	@Produces("application/json")
	public XMLItemCollection getWorkItemJSON(
			@PathParam("uniqueid") String uniqueid,
			@QueryParam("items") String items) {
		return getWorkItem(uniqueid,items);
	}

	

	

	/**
	 * Returns a file attachment located in the property $file of the spcified
	 * workitem
	 * 
	 * @param uniqueid
	 * @return
	 */
	@GET
	@Path("/workitem/{uniqueid}/file/{file}")
	public Response getWorkItemFile(@PathParam("uniqueid") String uniqueid,
			@PathParam("file") String file) {

		ItemCollection workItem;
		try {
			workItem = workflowService.getWorkItem(uniqueid);

			if (workItem != null) {
				// fetch $file from hashmap....

				HashMap mapFiles = null;
				Vector vFiles = workItem.getItemValue("$file");
				if (vFiles != null && vFiles.size() > 0) {
					mapFiles = (HashMap) vFiles.elementAt(0);

					Vector<Object> vectorFileInfo = new Vector<Object>();
					vectorFileInfo = (Vector) mapFiles.get(file);
					if (vectorFileInfo != null) {
						String sContentType = vectorFileInfo.elementAt(0)
								.toString();
						byte[] fileContent = (byte[]) vectorFileInfo
								.elementAt(1);

						// Set content type in order of the contentType stored
						// in the $file attribute
						Response.ResponseBuilder builder = Response.ok(
								fileContent, sContentType);

						return builder.build();

					} else
						// workitem not found
						return Response.status(Response.Status.NOT_FOUND)
								.build();

				} else
					// workitem not found
					return Response.status(Response.Status.NOT_FOUND).build();

			} else {
				// workitem not found
				return Response.status(Response.Status.NOT_FOUND).build();
			}

		} catch (Exception e) {

			e.printStackTrace();
		}

		return Response.status(Response.Status.NOT_FOUND).build();

	}

	/**
	 * This method post a ItemCollection object to be processed by the
	 * WorkflowManager. The method test for the propertys $processid and
	 * $activityid
	 * 
	 * @param workitem
	 *            - workitem data
	 */
	@PUT
	@Path("/workitem")
	@Consumes({ "application/xml", "text/xml" })
	public Response putWorkitem(XMLItemCollection workitem) {

		ItemCollection itemCollection;
		try {
			itemCollection = XMLItemCollectionAdapter
					.getItemCollection(workitem);
			workflowService.processWorkItem(itemCollection);
			return Response.status(Response.Status.OK).build();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return Response.status(Response.Status.NOT_ACCEPTABLE).build();
	}

	/**
	 * This method post a collection of ItemCollection objects to be processed
	 * by the WorkflowManager.
	 * 
	 * @param worklist
	 *            - workitem list data
	 */
	@PUT
	@Path("/workitems")
	@Consumes({ "application/xml", "text/xml" })
	public Response putWorkitems(EntityCollection worklist) {

		XMLItemCollection entity;
		ItemCollection itemCollection;
		try {
			// save new entities into database and update modelversion.....
			for (int i = 0; i < worklist.getEntity().length; i++) {
				entity = worklist.getEntity()[i];
				itemCollection = XMLItemCollectionAdapter
						.getItemCollection(entity);
				// process entity
				workflowService.processWorkItem(itemCollection);
			}
			return Response.status(Response.Status.OK).build();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return Response.status(Response.Status.NOT_ACCEPTABLE).build();
	}

	
	
	@POST
	@Path("/workitem")
	@Consumes({ "application/xml", "text/xml" })
	public Response postWorkitem(XMLItemCollection workitem) {
		return postWorkitem(workitem);
	}

	@POST
	@Path("/workitems")
	@Consumes({ "application/xml", "text/xml" })
	public Response postWorkitems(EntityCollection worklist) {
		return putWorkitems(worklist);
	}

	
	
	/**
	 * This method returns a List object from a given comma separated string.
	 * The method returns null if no elements are found.
	 * The provided parameter looks typical like this:
	 * <code>
	 *   txtWorkflowStatus,numProcessID,txtName
	 * </code>
	 * @param items
	 * @return
	 */
	private List<String> getItemList(String items) {
		if (items==null || "".equals(items))
			return null;
		Vector<String> v=new Vector<String>();
		StringTokenizer st=new StringTokenizer(items,",");
		while (st.hasMoreTokens())
			v.add(st.nextToken());
		return v;
	}
}
